import * as d3 from 'd3';

function renderData(dataElements) {

    let width = 500;
    let height = 400;
    let years = new Set(dataElements.map(o => parseInt(o.Year)));
    let states = new Set(dataElements.map(o => o.State));
    let data = dataElements.map(o => ({ year: parseInt(o.Year), state: o.State, rate: parseFloat(o.Abortion_Rate) }));

    let margin = ({ top: 25, right: 20, bottom: 35, left: 40 })
    let x = d3.scaleBand()
        .domain(years)
        .range([margin.left+1, width - margin.right]);

    let y = d3.scaleBand()
        .domain(states)
        .range([height - margin.bottom, margin.top]);

    // function with g arg assigned to xAxis
    let xAxis = g => g
        .attr("transform", `translate(0,${height - margin.bottom})`)
        .call(d3.axisBottom(x));

    // function with g arg assigned to yAxis
    let yAxis = g => g
        .attr("transform", `translate(${margin.left},0)`)
        .call(d3.axisLeft(y));

    const svg = d3.select("body").append("svg")
        .attr("viewBox", [-10, -20, width * 1.6, height * 1.1]);

    svg.append("g")
        .call(xAxis);

    svg.append("g")
        .call(yAxis);

    let colorZ = d3.scaleLinear()
        .range(["#efefef", "#ff0022"])
        .domain([0, 51]);

    svg.append("g")
        .selectAll()
        .data(data, d => { return d.year + ':' + d.state; })
        .enter()
        .append("rect")
        .attr("x", d => x(d.year))
        .attr("y", d => y(d.state))
        .attr("width", x.bandwidth())
        .attr("height", y.bandwidth())
        .style("fill", d => colorZ(d.rate));

    svg.append("g")
        .append("rect")
        .attr("y", "60")
        .attr("x", "510")
        .attr("width", "25")
        .attr("height", "25")
        .style("fill", "#efefef");

    svg.append("g")
        .append("rect")
        .attr("y", "90")
        .attr("x", "510")
        .attr("width", "25")
        .attr("height", "25")
        .style("fill", "#ff0022");

    svg.append('text')
        .attr("fill", "black")
        .attr("x", width - 350)
        .attr("y", height - 3)
        .style("font-size", "14px")
        .text("Years");

    svg.append("text")
        .attr("fill", "black")
        .attr("x", "150")
        .attr("y", "-15")
        .attr("transform", "translate(30, 400) rotate(270)")
        .style("font-size", "14px")
        .text("States");

    svg.append("text")
        .attr("fill", "black")
        .attr("x", width - 400)
        .attr("y", "5")
        .style("font-size", "18px")
        .text("Abortion Rates for Group 15-17 Years Old");

    svg.append("text")
        .attr("fill", "black")
        .attr("x", "500")
        .attr("y", "50")
        .style("font-size", "12px")
        .text("Abortion Rates per 1000:");

    svg.append("text")
        .attr("fill", "black")
        .attr("x", "550")
        .attr("y", "80")
        .style("font-size", "12px")
        .text("0");

    svg.append("text")
        .attr("fill", "black")
        .attr("x", "550")
        .attr("y", "110")
        .style("font-size", "12px")
        .text("51");
}

d3.csv(
    "https://gist.githubusercontent.com/olga-kondr/8c0189b181cf74dea61c96b669e60260/raw/c3ec5a222078115fcef9c98bcc9869a2e1d0b329/ab_rate.csv"
).then(data => renderData(data));